;;; Sierra Script 1.0 - (do not remove this comment)
(script# 103)
(include sci.sh)
(include game.sh)
(include 103.shm)
(include 103.shp)
(use Cycle)
(use Door)
(use Feature)
(use Game)
(use Main)
(use System)
(use Polygon)
(use View)
(use Scaler)
(use Sound)
(public
	rm103 0
)
(define TICKS_PER_CHANGE_WATER 4)
(define TICKS_PER_CHANGE_LAVA 7)
(define PALETTE_CHANGE_START 64)
(define PALETTE_CHANGE_LENGTH 31)
(define NEAR_SCALE_PERCENT 100)
(define FAR_SCALE_PERCENT 75)
(define NEAR_Y 129)
(define FAR_Y 110)
(local
	TicksPerChange =  TICKS_PER_CHANGE_WATER
	[Intensities 6] = [100 98 96 94 96 98]
	Offset =  0
	TickCount =  0
)
(instance rm103 of Room
	(properties
		picture scriptNumber
		north 0
		east 0
		south 0
		west 0
		noun N_ROOM
		style dpOPEN_FADEPALETTE
	)
	
	(method (init)
		(AddPolygonsToRoom @P_Default103)
		(super init:)
		(if (Btest F_PressedRedButton)
			(Palette palSET_FROM_RESOURCE 103)
			(= TicksPerChange TICKS_PER_CHANGE_LAVA)
		)
		(self setScript: RoomScript)
		(switch gPreviousRoomNumber
			(110 (SetUpEgo -1 5) (gEgo posn: 280 114))
			(else  (SetUpEgo -1 0) (gEgo posn: 33 173))
		; Set up ego view and loop (direction)
		)
		(gEgo
			init:
			setScale:
				Scaler
				NEAR_SCALE_PERCENT
				FAR_SCALE_PERCENT
				NEAR_Y
				FAR_Y
		)
		(greenButton approachVerbs: V_DO V_LOOK init:)
		(redButton approachVerbs: V_DO V_LOOK init:)
		(electricPost
			approachVerbs: V_DO V_LOOK V_DUMPSALTYMUG V_DUMPMUG
			init:
		)
		(if (Btest F_DestroyedElectricPost)
			(electricPost _case: C_OPEN setCel: 1)
			(greenButton _case: C_OPEN setLoop: 1)
			(redButton _case: C_OPEN setLoop: 1)
		)
		(if (not (Btest F_PickedUpMug))
			(mug approachVerbs: V_DO V_LOOK init:)
		)
		(if (not (Btest F_PickedUpSalt))
			(salt approachVerbs: V_DO V_LOOK init:)
		)
		(if (Btest F_PressedGreenButton)
			(secretDoor init: setCel: (secretDoor lastCel:))
		)
		(if (Btest F_PressedRedButton) (brookSound number: 3))
		(brookSound play:)
	)
)

(instance RoomScript of Script
	(properties)
	
	(method (doit &tmp index i start)
		(super doit:)
		; REVIEW: Do this another way. i.e. with a script. Counting ticks is more compilcated than this.
		(++ TickCount)
		(if (> TickCount TicksPerChange)
			(for ( (= i 0)) (< i 6)  ( (++ i)) (= start
				(+ PALETTE_CHANGE_START (* i PALETTE_CHANGE_LENGTH))
			) (= index (mod (+ i Offset) 6)) (Palette
				palSET_INTENSITY
				start
				(+ start PALETTE_CHANGE_LENGTH)
				[Intensities
				index]
			))
			(-= TickCount TicksPerChange)
			(-- Offset)
		)
		(if (& ctlNAVY (gEgo onControl:))
			(gEgo setScale: 0)
			(gRoom newRoom: 104)
		)
		(if (Btest F_PressedGreenButton)
			(if (& ctlGREEN (gEgo onControl:))
				(gEgo setScale: 0)
				(gRoom newRoom: 110)
			)
		)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0)
			; Handle state changes
			(1 (gGame handsOff:) (= seconds 2))
			; Destroying the electric box.                       ; TODO make sound
			(2
				(gMessager
					say: N_ELECTRICPOST V_DUMPSALTYMUG C_CLOSED 0 self
				)
			)
			(3
				(smoke init: setCycle: EndLoop self)
				(electricPost _case: C_OPEN setCel: 1)
				(greenButton _case: C_OPEN setLoop: 1)
				(redButton _case: C_OPEN setLoop: 1)
			)
			(4
				(smoke hide:)
				(gGame handsOn:)
				(ScoreFlag F_DestroyedElectricPost 5)
			)
		)
	)
)

(instance electricPost of Prop
	(properties
		view 3
		loop 2
		x 136
		y 137
		approachX 159
		approachY 151
		signal ignAct
		noun N_ELECTRICPOST
		_case C_CLOSED
	)
	
	(method (doVerb theVerb)
		(if (== _case C_CLOSED)
			(switch theVerb
				(V_DUMPMUG
					(super doVerb: theVerb &rest)
					(gEgo put: INV_FULLMUG)
					(gEgo get: INV_EMPTYMUG)
				)
				(V_DUMPSALTYMUG
					(gEgo put: INV_SALTYFULLMUG)
					(gEgo get: INV_EMPTYMUG)
					(RoomScript cue:)
				)
				(else  (super doVerb: theVerb &rest))
			)
		else
			(super doVerb: theVerb &rest)
		)
	)
)

(instance greenButton of Prop
	(properties
		view 3
		loop 0
		cel 1
		x 185
		y 164
		z 10
		approachX 162
		approachY 162
		signal ignAct
		noun N_GREENBUTTON
		_case C_CLOSED
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(cond 
					((== _case C_CLOSED) (super doVerb: theVerb &rest))
					((Btest F_PressedGreenButton) (gMessager say: N_ROOM 0 C_NOTHING))
					(else
						; Open the secret door
						(ScoreFlag F_PressedGreenButton 1)
						(secretDoor init: setCycle: EndLoop)
						(openDoorSound play:)
					)
				)
			)
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance redButton of Prop
	(properties
		view 3
		loop 0
		cel 0
		x 208
		y 177
		z 10
		approachX 183
		approachY 175
		signal ignAct
		noun N_REDBUTTON
		_case C_CLOSED
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(cond 
					((== _case C_CLOSED) (super doVerb: theVerb &rest))
					((Btest F_PressedRedButton) (gMessager say: N_ROOM 0 C_NOTHING))
					(else
						; Cause hell fire.
						(ScoreFlag F_PressedRedButton 1)
						(self setScript: causeHellfireScript)
					)
				)
			)
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance causeHellfireScript of Script
	(properties)
	
	(method (doit)
		(super doit:)
		(if (== 64 (PalVary pvGET_CURRENT_STEP))
			(PalVary pvUNINIT)
			(= TicksPerChange TICKS_PER_CHANGE_LAVA)
			(gGame handsOn:)
			(brookSound stop: number: 3 play:)
			(self dispose:)
		)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (gGame handsOff:) (PalVary pvINIT 103 1 64 1))
		)
	)
)

(instance mug of Prop
	(properties
		view 901
		loop 2
		x 262
		y 165
		approachX 242
		approachY 162
		signal ignAct
		noun N_MUG
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(ScoreFlag F_PickedUpMug 2)
				(gEgo get: INV_EMPTYMUG)
				(self hide:)
			)
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance salt of Prop
	(properties
		view 902
		loop 0
		x 257
		y 144
		approachX 237
		approachY 144
		signal ignAct
		noun N_SALT
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(ScoreFlag F_PickedUpSalt 2)
				(gEgo get: INV_SALT)
				(self hide:)
			)
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance smoke of Prop
	(properties
		view 103
		x 138
		y 100
		signal ignAct
	)
)

(instance secretDoor of Prop
	(properties
		view 104
		x 294
		y 99
		signal ignAct
		noun N_DOORWAY
	)
)

(instance brookSound of Sound
	(properties
		number 1
		priority 15
		flags 1
		loop -1
	)
)

(instance openDoorSound of Sound
	(properties
		number 103
		priority 15
		flags 1
	)
)
