;;; Sierra Script 1.0 - (do not remove this comment)
(script# 104)
(include sci.sh)
(include game.sh)
(include 104.shm)
(include 104.shp)
(use Cycle)
(use Door)
(use Feature)
(use Game)
(use Main)
(use System)
(use Polygon)
(use View)
(use Sound)
(use Scaler)
(public
	rm104 0
)
(define NEAR_SCALE_PERCENT 100)
(define FAR_SCALE_PERCENT 25)
(define NEAR_Y 168)
(define FAR_Y 143)
(local
	[FireX 4] = [149 242 142 206]
	[FireY 4] = [156 156 171 161]
	[Fire2X 4] = [178 228 219]
	[Fire2Y 4] = [114 86 120]
)
(instance rm104 of Room
	(properties
		picture scriptNumber
		north 0
		east 0
		south 0
		west 0
		noun N_ROOM
	)
	
	(method (init &tmp theCase)
		(self setRegions: FLOPPYDISK_REGION)
		(AddPolygonsToRoom @P_Default104)
		(super init:)
		(if (Btest F_PressedRedButton)
			(Palette palSET_FROM_RESOURCE 104)
			; Sky overlay... this will be overlayed on top of the current pic.
			(self overlay: 1104)
			(= theCase C_FIRE)
		else
			(= theCase C_WATER)
		)
		(self _case: theCase)
		(self setScript: RoomScript)
		(switch gPreviousRoomNumber
			(103 (SetUpEgo -1 1) (gEgo posn: 157 135))
			; Set up ego view and loop (direction)
			(else 
				(gGame handsOff:)
				(SetUpEgo 1 0)
				(gEgo setCel: (gEgo lastCel:))
				(gEgo posn: 237 185)
				(gEgo setScript: TransportInScript)
			)
		)
		(gEgo
			init:
			setScale:
				Scaler
				NEAR_SCALE_PERCENT
				FAR_SCALE_PERCENT
				NEAR_Y
				FAR_Y
		)
		(waterfall
			_case: theCase
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Waterfall)
			init:
		)
		(lake
			_case: theCase
			approachVerbs: V_FILLMUG V_DO
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Lake)
			init:
		)
		(if (Btest F_PressedRedButton)
			(lightning init:)
			(brookSound number: 4)
			(poof init:)
			(poof2 init:)
		else
			(spray init: setCycle: Forward)
		)
		(brookSound play:)
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_LOOK
				(if (> (gEgo y?) 167)
					; Specify 0 for the sequence, so we do the entire sequence
					(gMessager say: noun theVerb _case 0 0 scriptNumber)
					; For the sequence to work, 1 must be returned.
					(return 1)
				else
					(gMessager say: noun theVerb _case 1 0 scriptNumber)
					(return 1)
				)
			)
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance TransportInScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (gEgo setCycle: BegLoop self))
			(1 (SetUpEgo 0 3) (gEgo init:) (gGame handsOn:))
		)
	)
)

(instance RoomScript of Script
	(properties)
	
	(method (doit)
		(super doit:)
		(Palette palANIMATE 192 198 -9)
		(Palette palANIMATE 198 204 -9)
		(Palette palANIMATE 204 210 -9)
		(Palette palANIMATE 210 216 -9)
		(Palette palANIMATE 216 222 -9)
		(Palette palANIMATE 222 228 -9)
		(Palette palANIMATE 227 235 -9)
		; Use point, since bounding rect is still full size, even though we're scaled small.
		(if (& ctlNAVY (gEgo onControl: TRUE))
			(gRoom newRoom: 103)
			(gEgo setScale: 0)
		)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0)
		)
	)
)

; Handle state changes
(instance brookSound of Sound
	(properties
		number 2
		priority 15
		flags 1
		loop -1
	)
)

(instance waterfall of Feature
	(properties
		x 202
		y 88
		noun N_WATERFALL
	)
)

(instance lake of Feature
	(properties
		x 135
		y 149
		noun N_LAKE
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_FILLMUG
				(ScoreFlag F_FilledMugWithWater 5)
				; TODO: Do we need to change the cursor??? Or is that taken care of?
				(gEgo put: INV_EMPTYMUG)
				(gEgo get: INV_FULLMUG)
				(gMessager say: N_LAKE V_FILLMUG _case)
				(return 1)
			)
			(else  (super doVerb: theVerb &rest))
		)
	)
	
	(method (handleEvent pEvent)
		; If the player clicked on the lake for a verb, then set the approach point to that point.
		(if
			(and
				(& (pEvent type?) evVERB)
				(self onMe: pEvent)
				(self isNotHidden:)
			)
			(self approachX: (pEvent x?) approachY: (pEvent y?))
		)
		(super handleEvent: pEvent)
	)
)

(instance lightning of Prop
	(properties
		view 105
		x 150
		y 100
		priority 0
		signal (| ignAct fixPriOn)
	)
	
	(method (init)
		(super init: &rest)
		(self setScript: lightningScript)
	)
)

(instance lightningScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (lightning hide:) (= seconds (Random 1 3)))
			(1
				(lightning
					setCel: (Random 0 4)
					posn: (Random 76 144) 63
					show:
				)
				(= ticks 10)
			)
			(2 (lightning hide:) (self changeState: 0))
		)
	)
)

(instance poof of Prop
	(properties
		x 132
		y 154
		view 106
		cycleSpeed 3
		signal ignAct
	)
	
	(method (init)
		(super init: &rest)
		(self setScript: poofScript)
	)
)

(instance poofScript of Script
	(properties)
	
	(method (changeState newState &tmp spot)
		(= state newState)
		(switch state
			(0 (client hide:) (= seconds (Random 3 6)))
			(1
				(= spot (Random 0 4))
				(client
					posn: [FireX spot] [FireY spot]
					setCel: 0
					show:
					setCycle: EndLoop self
				)
			)
			(2 (self changeState: 0))
		)
	)
)

(instance poof2 of Prop
	(properties
		x 150
		y 170
		priority 11
		view 108
		cycleSpeed 3
		signal (| fixPriOn ignAct)
	)
	
	(method (init)
		(super init: &rest)
		(self setScript: poof2Script)
	)
)

(instance poof2Script of Script
	(properties)
	
	(method (changeState newState &tmp spot)
		(= state newState)
		(switch state
			(0 (client hide:) (= seconds (Random 2 7)))
			(1
				(= spot (Random 0 3))
				(client
					posn: [Fire2X spot] [Fire2Y spot]
					setCel: 0
					show:
					setCycle: EndLoop self
				)
			)
			(2 (self changeState: 0))
		)
	)
)

(instance spray of Prop
	(properties
		view 109
		x 183
		y 149
		signal ignAct
	)
)
