;;; Sierra Script 1.0 - (do not remove this comment)
(script# 105)
(include sci.sh)
(include game.sh)
(include 105.shm)
(include 105.shp)
(use Controls)
(use Cycle)
(use Door)
(use View)
(use Game)
(use Main)
(use System)
(use Sound)
(use Feature)
(public
	rm105 0
)
(define ROOM_PALETTE_START 0)
(define ROOM_PALETTE_END 190)
(define SCREEN_PALETTE_START 191)
(define SCREEN_PALETTE_END 255)
(define SCREEN_OFF_INTENSITY 10)
(define SCREEN_ON_INTENSITY 100)
(define SCREEN_INTENSITY_CHANGE 6)
(define SCREEN_INTENSITY_WOBBLE 99)
(define WOBBLE_PERIOD 4)
(define TICKS_BEFORE_TRANSPORT 400)
(local
	IsScreenOn =  FALSE
	IsLightOn =  FALSE
	PaletteCycleSpeed =  0
	CurrentScreenIntensity =  SCREEN_OFF_INTENSITY
	WobbleCountdown =  WOBBLE_PERIOD
	TicksCount =  0
	LastTicksCount =  0
)

(procedure (UpdateRoomLight &tmp targetPalette)
	(PalVary pvUNINIT)
	(cond 
		(IsLightOn (= targetPalette 105))
		(IsScreenOn (= targetPalette 107))
		; Normal
		; Lit up by orange pink glow
		(else (= targetPalette 106))
	; Dark...
	)
	; 64 indicates a complete transition to the new palette
	; (32 would be halfway). 64 is the default.
	(PalVary pvINIT targetPalette 1 64 3)
)

(instance rm105 of Room
	(properties
		picture 105
		north 0
		horizon 50
		east 0
		south 106
		west 0
		noun N_ROOM
		style dpOPEN_INSTANTLY
	)
	
	(method (init)
		(AddPolygonsToRoom @P_Default105)
		
		; There is some weird palette glitching here (upon entering from the south) which I'm not sure how to fix.
		(super init:)
		(self setScript: RoomScript)

		(Palette palSET_FROM_RESOURCE 106)
		(Palette
			palSET_INTENSITY
			SCREEN_PALETTE_START
			SCREEN_PALETTE_END
			10
		)		

		(switch gPreviousRoomNumber
			(else  (SetUpEgo -1 3) (gEgo posn: 63 178))
		)
		(gEgo init:)
		(UpdateRoomLight)
		(lightSwitch1 approachVerbs: V_DO init:)
		(screenButton approachVerbs: V_DO init:)
		(screen init:)
		(dirtyArea init:)
		(destination init: hide:)
	)
	
	(method (newRoom newRoomNumber)
		; We need to uninitialize any palette transitions before we exit the room.
		(PalVary pvUNINIT)
		; In case the drone is still playing...
		(gMusic1 stop:)
		(super newRoom: newRoomNumber)
	)
)

(instance RoomScript of Script
	(properties)
	
	(method (doit &tmp onControl exitingScript)
		(super doit:)
		(= onControl (gEgo onControl:))
		(cond 
			((& onControl ctlNAVY) (= PaletteCycleSpeed -1))
			((& onControl ctlGREEN) (= PaletteCycleSpeed -2))
			((& onControl ctlTEAL) (= PaletteCycleSpeed -3))
			(else (= PaletteCycleSpeed 0))
		)
		(= exitingScript NULL)
		(if (and IsScreenOn (!= PaletteCycleSpeed 0))
			(ScoreFlag F_WalkedToTV 2)
			; A cycle speed or 1 or -1 is fastest. Larger numbers are slower.
			(Palette
				palANIMATE
				SCREEN_PALETTE_START
				SCREEN_PALETTE_END
				PaletteCycleSpeed
			)
			(if (== PaletteCycleSpeed -1)
				(= exitingScript exitThroughTV)
			)
		)
		(if (!= exitingScript (self script?))
			(self setScript: exitingScript)
		)
	)
	
	(method (handleEvent pEvent)
		(super handleEvent: pEvent)
	)
)

(instance exitThroughTV of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (= ticks TICKS_BEFORE_TRANSPORT))
			(1 (destination setScript: destinationAppears self))
			(2
				; Transport out of room
				(transportSound play:)
				(gGame handsOff:)
				(gEgo view: 1 cel: 0 setCycle: EndLoop self)
			)
			(3
				(gGame handsOn:)
				(if IsLightOn
					(gRoom newRoom: 104)
				else
					(gRoom newRoom: 400)
				)
			)
		)
	)
	
	(method (dispose)
		(super dispose:)
		(destination hide: setScript: NULL)
	)
)

(instance lightSwitch1 of Prop
	(properties
		x 70
		y 112
		z 46
		view 100
		cel 0
		approachDist 30
		approachX 85
		approachY 110
		noun N_LIGHTSWITCH
		signal ignAct
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(if IsLightOn
					(= IsLightOn FALSE)
					(UpdateRoomLight)
				else
					(= IsLightOn TRUE)
					(UpdateRoomLight)
					(ScoreFlag F_TurnedOnLight 1)
				)
				(lightSwitchSound play:)
			)
			(else  (super doVerb: theVerb))
		)
	)
)

(instance screenButton of Prop
	(properties
		x 110
		; y determines the priority of clicking on items. We want the button to be
		; higher priority
		y 112
		z 45
		view 100
		cel 1
		approachX 90
		; ... than the ego:
		approachY 110
		; approachY 105
		noun N_SCREENSWITCH
		priority 1
		signal (| fixPriOn ignAct)
	)
	
	(method (init)
		(super init:)
		(self setScript: turnScreenOnOffScript)
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(lightSwitchSound play:)
				(if IsScreenOn
					(= IsScreenOn FALSE)
					(UpdateRoomLight)
					(gMusic1 stop:)
				else
					(= IsScreenOn TRUE)
					(UpdateRoomLight)
					(gMusic1 number: 101 loop: 1 play: hold:)
					(ScoreFlag F_TurnedOnTV 1)
				)
			)
			(else  (super doVerb: theVerb))
		)
	)
)

(instance turnScreenOnOffScript of Script
	(properties)
	
	(method (doit &tmp otherIntensity)
		(-- WobbleCountdown)
		(if IsScreenOn
			(+= CurrentScreenIntensity SCREEN_INTENSITY_CHANGE)
		else
			(-= CurrentScreenIntensity SCREEN_INTENSITY_CHANGE)
		)
		(= CurrentScreenIntensity
			(Max CurrentScreenIntensity SCREEN_OFF_INTENSITY)
		)
		(= CurrentScreenIntensity
			(Min CurrentScreenIntensity SCREEN_ON_INTENSITY)
		)
		; The "to" parameter is exclusive
		(Palette
			palSET_INTENSITY
			SCREEN_PALETTE_START
			SCREEN_PALETTE_END
			CurrentScreenIntensity
		)
		(if (== 0 WobbleCountdown)
			(= otherIntensity 100)
			(if
				(and
					(!= PaletteCycleSpeed 0)
					(not IsLightOn)
					(== CurrentScreenIntensity SCREEN_ON_INTENSITY)
				)
				; Light is off, but screen is on, and is cycling. Wobble the other intensities
				(= otherIntensity (Random SCREEN_INTENSITY_WOBBLE 100))
			)
			(Palette
				palSET_INTENSITY
				ROOM_PALETTE_START
				SCREEN_PALETTE_START
				otherIntensity
			)
			(= WobbleCountdown WOBBLE_PERIOD)
		)
		; Update our noun
		(if IsScreenOn
			(if (!= PaletteCycleSpeed 0)
				(screen noun: N_SCREENMOVING)
			else
				(screen noun: N_SCREENON)
			)
		else
			(screen noun: N_SCREEN)
		)
	)
)

(instance lightSwitchSound of Sound
	(properties
		priority 15
		number 105
		loop 1
		flags 1
	)
)

(instance transportSound of Sound
	(properties
		priority 15
		number 403
		loop 1
		flags 1
	)
)

(instance screen of Feature
	(properties
		x 210
		y 50
		nsTop 14
		nsLeft 162
		nsBottom 90
		nsRight 256
		noun N_SCREEN
	)
)

(instance dirtyArea of Feature
	(properties
		x 216
		y 110
		noun N_DIRTYAREA
	)
	
	(method (init)
		(= onMeCheck (CreateNewPolygon @P_DirtyArea))
		(super init: &rest)
	)
)

(instance destination of Prop
	(properties
		view 110
		x 209
		y 90
		signal ignAct
		cycleSpeed 10
	)
)

; A script that controls the teleport destination gradually flickering in.
(instance destinationAppears of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(client loop: (if IsLightOn 0 else 1) show: setCel: 0)
				(= ticks 10)
			)
			(1 (client setCycle: CycleTo 1 1 self))
			(2 (client setCycle: CycleTo 0 -1 self))
			(3 (= seconds (Random 1 2)))
			(4 (client setCycle: CycleTo 3 1 self))
			(5 (client setCycle: CycleTo 0 -1 self))
			(6 (= seconds (Random 1 2)))
			(7 (client setCycle: CycleTo 4 1 self))
			(8 (client setCycle: CycleTo 0 -1 self))
			(9 (= seconds (Random 1 2)))
			(10 (client setCycle: CycleTo 5 1 self))
			(11 (self dispose:))
		)
	)
)
