;;; Sierra Script 1.0 - (do not remove this comment)
(script# 111)
(include sci.sh)
(include game.sh)
(include 111.shm)
(include 111.shp)
(use Cycle)
(use Door)
(use Feature)
(use Game)
(use Main)
(use System)
(use Polygon)
(use View)
(use Sound)
(use Print)
(public
	rm111 0
)
(define SCREEN_LEFT 45)
(define SCREEN_TOP 31)
(define DRAG_MARGIN 10)
(define PIECE_CX 43)
(define PIECE_CY 30)
(define PUZZLE_PIECE_COUNT 9)
(define SCREEN_PALETTE_START 224)
(define SCREEN_PALETTE_END 255)
(define PIECE_PRI_BACKGROUND 0)
(define PIECE_PRI_FOREGROUND 1)
(local
	Loaded =  FALSE
	TrackPiece
	DragBoundsLeft
	DragBoundsTop
	DragBoundsRight
	DragBoundsBottom
	DragOffsetX
	DragOffsetY
	[takenSpots 9]
	[pieceProps 9]
	ScreenCondition =  C_NOPIECES
)
; Create and place the puzzle pieces. Any new piece goes in the first empty slot
; The player has this piece.
; This is the first time this piece has been seen. Find a spot for it.
; Put the pieces at their initial locations
; Eject button
; Polygons don't seem to to work properly for 200px high pics. The mouse event seems to shift position after first click.
; setOnMeCheck(omcPOLYGON CreateNewPolygon(@P_Eject))
; Hmm, control colors don't work reliably either... But sometimes they do.
(procedure (CalculateTakenSpots &tmp i)
	(for ( (= i 0)) (< i PUZZLE_PIECE_COUNT)  ( (++ i)) (= [takenSpots  i] FALSE) (if (!= [gPuzzlePieceSlot i] -1)
		(= [takenSpots  [gPuzzlePieceSlot i]] TRUE)
	))
)

(procedure (GetPieceInSlot slot &tmp i)
	(for ( (= i 0)) (< i PUZZLE_PIECE_COUNT)  ( (++ i)) (if (== [gPuzzlePieceSlot i] slot)
		(return [pieceProps i])
	))
	(return NULL)
)

(procedure (FindFirstEmpty &tmp spot)
	(= spot 0)
	(while (< spot PUZZLE_PIECE_COUNT)
		(if (not [takenSpots spot]) (return spot))
		(++ spot)
	)
	(DebugPrint {error})
	(return 0)
)

(procedure (GetPieceX order)
	(return (+ SCREEN_LEFT (* PIECE_CX (mod order 3))))
)

(procedure (GetPieceY order)
	(return (+ SCREEN_TOP (* PIECE_CY (/ order 3))))
)

(procedure (IsPuzzleSolved &tmp i)
	(for ( (= i 0)) (< i 9)  ( (++ i))
		;(DebugPrint {i: %d g[i]: %d} i [gPuzzlePieceSlot i])
		(if (!= [gPuzzlePieceSlot i] i) (return FALSE))
	)
	(return TRUE)
)

; Move the piece with the mouse.
; Ensure it's in bounds.
; Now the screen opens.
; Disable all icons except look and do		
(procedure (MapPosToSlot piece &tmp x y row col)
	; Add half size to thing, then subtract and divide.
	(= x (piece x?))
	(= y (piece y?))
	(+= x (/ PIECE_CX 2))
	(+= y (/ PIECE_CY 2))
	(-= x SCREEN_LEFT)
	(-= y SCREEN_TOP)
	(= col (Min 2 (/ x PIECE_CX)))
	(= row (Min 2 (/ y PIECE_CY)))
	(return (+ (* row 3) col))
)

(procedure (SnapToGrid pieceId &tmp piece)
	(= piece [pieceProps pieceId])
	(piece
		posn:
			(GetPieceX [gPuzzlePieceSlot pieceId])
			(GetPieceY [gPuzzlePieceSlot pieceId])
	)
)

; Place the current TrackPiece, swapping with an exiting piece in the same slot if necessary.
(procedure (PlacePiece &tmp slot existingPiece pieceId sourceSlot)
	(= pieceId (TrackPiece cel?))
	(= slot (MapPosToSlot TrackPiece))
	; If there is another in this slot, then swap
	(= existingPiece (GetPieceInSlot slot))
	(cond 
		((== existingPiece TrackPiece) (SnapToGrid pieceId))
		; Nothing to do except reposition it
		((== existingPiece NULL) (= [gPuzzlePieceSlot  pieceId] slot) (SnapToGrid pieceId))
		; Place it, there's nothing there
		(else
			; Need to swap
			(= sourceSlot [gPuzzlePieceSlot pieceId])
			(= [gPuzzlePieceSlot  (existingPiece cel?)] sourceSlot)
			(= [gPuzzlePieceSlot  pieceId] slot)
			(SnapToGrid (existingPiece cel?))
			(SnapToGrid pieceId)
		)
	)
)

(instance rm111 of Room
	(properties
		picture scriptNumber
		north 0
		east 0
		south 0
		west 0
		style dpOPEN_FADEPALETTE
		noun N_ROOM
	)
	
	(method (init &tmp pieceCount i firstSpot)
		(gIconBar hide: disable:)
		(SetPort 0 0 200 320 0 0)
		(AddPolygonsToRoom @P_Default111)
		(= DragBoundsLeft (- SCREEN_LEFT DRAG_MARGIN))
		(= DragBoundsTop (- SCREEN_TOP DRAG_MARGIN))
		(= DragBoundsRight
			(+ (+ SCREEN_LEFT (* 2 PIECE_CX)) DRAG_MARGIN)
		)
		(= DragBoundsBottom
			(+ (+ SCREEN_TOP (* 2 PIECE_CY)) DRAG_MARGIN)
		)
		(super init:)
		(self setScript: RoomScript)
		(floppyInsert init:)
		(for ( (= i 0)) (< i PUZZLE_PIECE_COUNT)  ( (++ i)) (if (Btest (+ F_Puzzle_First i))
			(if (== [gPuzzlePieceSlot i] -1)
				(CalculateTakenSpots)
				(= [gPuzzlePieceSlot  i] (FindFirstEmpty))
			)
		))
		(for ( (= i 0)) (< i PUZZLE_PIECE_COUNT)  ( (++ i)) (if (!= [gPuzzlePieceSlot i] -1)
			(= [pieceProps  i]
				((PuzzlePiece new:)
					signal: ignAct
					view: 112
					cel: i
					posn:
						(GetPieceX [gPuzzlePieceSlot i])
						(GetPieceY [gPuzzlePieceSlot i])
					init:
					yourself:
				)
			)
			(= ScreenCondition C_SOMEPIECES)
		))
		(ejectButton init: setOnMeCheck: omcCOLORS ctlTEAL)
		(theKeyboard init: setOnMeCheck: omcCOLORS ctlNAVY)
		(theScreen init: setOnMeCheck: omcCOLORS ctlGREEN)
	)
	
	(method (dispose)
		(if (!= gNewRoomNumber 1111)
			; unless we're going to the final room, restore the status bar and such...
			(SetPort 0 0 190 320 10 0)
			(gIconBar hide: enable:)
		)
		(super dispose: &rest)
	)
)

(instance RoomScript of Script
	(properties)
	
	(method (doit &tmp newX newY)
		(if (and (== state 2) (== (PalVary pvGET_CURRENT_STEP) 64))
			(self cue:)
		)
		(if (IsObject TrackPiece)
			(= newX (- gPEventX DragOffsetX))
			(= newY (- gPEventY DragOffsetY))
			(= newX (Max newX DragBoundsLeft))
			(= newX (Min newX DragBoundsRight))
			(= newY (Max newY DragBoundsTop))
			(= newY (Min newY DragBoundsBottom))
			(TrackPiece x: newX)
			(TrackPiece y: newY)
		)
		(if (and Loaded (IsPuzzleSolved))
			(gRoom setScript: PuzzleSolvedScript)
		)
		(super doit:)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (gGame handsOff:) (= seconds 2))
			(1 (diskSound play:) (floppyInsert setCycle: EndLoop self))
			(2
				(floppyInsert hide:)
				(computerLight init: setCycle: Forward)
				(computerFan play:)
				(PalVary pvINIT 112 2 64 1)
			)
			(3
				(PalVary pvUNINIT)
				(gGame handsOn:)
				(= Loaded TRUE)
				(if (== ScreenCondition C_NOPIECES)
					(Display
						{No data found.}
						dsCOORD
						45
						31
						dsFONT
						1
						dsCOLOR
						43
					)
				)
				(gIconBar
					disable: ICONINDEX_WALK ICONINDEX_TALK ICONINDEX_CUSTOM
				)
			)
		)
	)
)

(instance floppyInsert of Prop
	(properties
		view 111
		loop 0
		cel 0
		x 249
		y 104
		cycleSpeed 8
		signal ignAct
		noun N_FLOPPY
	)
)

(instance ejectFloppyScript of Script
	(properties)
	
	(method (doit)
		(if (and (== state 0) (== (PalVary pvGET_CURRENT_STEP) 64))
			(self cue:)
		)
		(super doit:)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(gGame handsOff:)
				(computerFan stop:)
				(computerFanOff play:)
				(computerLight hide:)
				(PalVary pvINIT 113 2 64 1)
			)
			(1
				(PalVary pvUNINIT)
				(floppyInsert show:)
				(floppyInsert setCel: (floppyInsert lastCel:))
				(floppyInsert setCycle: BegLoop self)
				(diskSound play:)
			)
			(2 (= seconds 2))
			(3 (gGame handsOn:) (gRoom newRoom: 110))
		)
	)
)

(instance diskSound of Sound
	(properties
		priority 15
		number 109
	)
)

(instance computerFan of Sound
	(properties
		priority 15
		loop -1
		number 110
	)
)

(instance computerFanOff of Sound
	(properties
		priority 15
		loop 1
		number 111
	)
)

(instance ejectButton of Feature
	(properties
		x 233
		y 110
		noun N_EJECT
	)
	
	(method (doVerb theVerb)
		(if (or (== theVerb V_DO) (== theVerb V_FLOPPYDISK))
			(rm111 setScript: ejectFloppyScript)
		else
			(super doVerb: theVerb &rest)
		)
	)
)

(instance computerLight of Prop
	(properties
		view 111
		loop 1
		signal ignAct
		x 232
		y 111
		cycleSpeed 10
	)
)

(instance theKeyboard of Feature
	(properties
		noun N_KEYBOARD
	)
)

(instance theScreen of Feature
	(properties
		noun N_SCREEN
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO (gMessager say: noun theVerb ScreenCondition))
			(else  (super doVerb: theVerb &rest))
		)
	)
)

; Add itself as an event handler when the DO verb is applied to it,
; and then waits for mouse release to remove itself.
(class PuzzlePiece of Prop
	(properties
		priority PIECE_PRI_BACKGROUND
		signal (| ignAct fixPriOn)
		noun N_PIECE
	)
	
	(method (handleEvent pEvent)
		(if (== evMOUSERELEASE (pEvent type?))
			(PlacePiece)
			(= TrackPiece NULL)
			(self setPri: PIECE_PRI_BACKGROUND)
			(pEvent claimed: 1)
			(gOldMH delete: self)
			(return 1)
		)
		(return (super handleEvent: pEvent))
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(if (== TrackPiece NULL)
					; Start tracking this one
					(= TrackPiece self)
					(= DragOffsetX (- gPEventX x))
					(= DragOffsetY (- gPEventY y))
					(self setPri: PIECE_PRI_FOREGROUND)
					(gOldMH addToFront: self)
				)
			)
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance PuzzleSolvedScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(gGame handsOff:)
				(= seconds 2)
			)
			(1
				(gRoom newRoom: 1111)
			)
		)
	)
)
