;;; Sierra Script 1.0 - (do not remove this comment)
(script# 200)
(include sci.sh)
(include game.sh)
(include 200.shm)
(include 200.shp)
(use Controls)
(use Cycle)
(use Door)
(use Feature)
(use Game)
(use Main)
(use System)
(use Polygon)
(use View)
(use Sound)
(public
	rm200 0
)
(local
	IsFireHot =  TRUE
	InFire =  FALSE
)
(instance rm200 of Room
	(properties
		picture scriptNumber
		north 0
		east 0
		south 0
		west 0
		noun N_ROOM
		style dpOPEN_INSTANTLY
	)
	
	(method (init)
		(AddPolygonsToRoom @P_Default200)
		(self setRegions: FLOPPYDISK_REGION)
		(ScoreFlag F_TransitionRoom 1)
		(if (== gPreviousRoomNumber 135)
			(self setScript: EnterToPathingScript)
		else
			(self setScript: RoomScript)
		)
		(super init:)
		(SetUpEgo)
		(switch gPreviousRoomNumber
			(135 (gEgo posn: 35 100 loop: 0))
			(else  (gEgo posn: 150 100 loop: 1))
		)
		(gEgo init:)
		(fire
			init:
			setCycle: Forward
			setOnMeCheck: omcCOLORS ctlYELLOW
		)
		(lightSwitch1 approachVerbs: V_DO init:)
		(finalTarget setPri: 15 setCycle: Forward init:)
		(nextTarget setCycle: Forward init:)
		(totalPolygon
			init:
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Total)
		)
		(nearestPolygon
			init:
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Nearest)
		)
		(barredPolygon
			init:
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Barred)
		)
		(containedPolygon
			init:
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Contained)
		)
	)
)

(instance RoomScript of Script
	(properties)
	
	(method (doit &tmp inFire theMover)
		(super doit:)
		(if (& ctlCYAN (gEgo onControl:))
			(rm200 setScript: ExitingFromPathingScript)
		)
		(= inFire (fire onMe: gEgo))
		(cond 
			(inFire
				(if (not InFire)
					(= InFire TRUE)
					(if (== (fire noun?) N_FIRE)
						(gMessager say: N_FIRE V_STAND 0 0 0)
					else
						; Transport back to the starting room
						(rm200 setScript: WaitThenExitScript)
						(teleportSound play: WaitThenExitScript)
					)
				)
			)
			(InFire (= InFire FALSE))
		)
		(= theMover (gEgo mover?))
		(if (IsObject theMover)
			(nextTarget show:)
			(finalTarget show:)
			(nextTarget posn: (theMover x?) (theMover y?))
			(finalTarget
				posn: (theMover finalX?) (theMover finalY?)
			)
		else
			(nextTarget hide:)
			(finalTarget hide:)
		)
	)
	
	(method (handleEvent pEvent)
		(super handleEvent: pEvent)
	)
	
	; handle Said's, etc...
	(method (changeState newState)
		(= state newState)
		(switch state
			(0)
		)
	)
)

; Handle state changes
(instance ExitingFromPathingScript of Script
	(properties)
	
	(method (doit)
		(super doit:)
		(if (>= (PalVary pvGET_CURRENT_STEP) 64)
			(PalVary pvUNINIT)
			(gGame handsOn:)
			(gRoom newRoom: 135)
		)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(exitEnterSound number: 135 play:)
				(gGame handsOff:)
				; Start the palette transition
				(PalVary pvINIT 888 1)
			)
		)
	)
)

(instance EnterToPathingScript of Script
	(properties)
	
	(method (doit)
		(super doit:)
		(if (== state 1)
			(if (== 0 (PalVary pvGET_CURRENT_STEP))
				(gGame handsOn:)
				(PalVary pvUNINIT)
				(rm200 setScript: RoomScript)
			)
		)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(exitEnterSound number: 136 play:)
				(gGame handsOff:)
				; 0 ticks means instantly go to palette 888, the white one.
				(PalVary pvINIT 888 0 64 1)
				; Before going in reverse, wait a few cycles for things to take effect (?)
				(= cycles 5)
			)
			(1 (PalVary pvREVERSE 1 0 1))
		; Step stop is 0. Back to the original palette.
		; We supply 1 for the direction, since it will be negated (because reverse)
		; 0 for the StepStop (the beginning)
		)
	)
)

(instance fire of Prop
	(properties
		x 182
		y 94
		view 200
		noun N_FIRE
		signal ignAct
	)
)

(instance lightSwitch1 of Prop
	(properties
		x 288
		y 50
		z 20
		view 100
		cel 0
		approachDist 30
		approachX 264
		approachY 59
		noun N_LIGHTSWITCH
		signal ignAct
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO
				(if IsFireHot
					(= IsFireHot FALSE)
					(fire noun: N_FIRECOLD view: 201)
				else
					(= IsFireHot TRUE)
					(fire noun: N_FIRE view: 200)
				)
				(lightSwitchSound play:)
			)
			(else  (super doVerb: theVerb))
		)
	)
)

(instance lightSwitchSound of Sound
	(properties
		priority 15
		number 105
		loop 1
		flags 1
	)
)

(instance teleportSound of Sound
	(properties
		priority 15
		number 200
		loop 1
		flags 1
	)
)

(instance WaitThenExitScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (gGame handsOff:))
			(1
				(ScoreFlag F_TransitionRoomExit 1)
				(gRoom newRoom: 110)
			)
		)
	)
)

(instance finalTarget of Prop
	(properties
		x 0
		y 0
		view 202
		loop 0
		signal ignAct
	)
)

(instance nextTarget of Prop
	(properties
		x 0
		y 0
		view 202
		loop 1
		signal ignAct
	)
)

(instance exitEnterSound of Sound
	(properties
		number 135
		priority 15
		loop 1
		flags 1
	)
)

(instance totalPolygon of Feature
	(properties
		noun N_TOTAL
		x 187
		y 111
	)
)

(instance nearestPolygon of Feature
	(properties
		noun N_NEAREST
		x 240
		y 147
	)
)

(instance barredPolygon of Feature
	(properties
		noun N_BARRED
		x 99
		y 97
	)
)

(instance containedPolygon of Feature
	(properties
		noun N_CONTAINED
		x 176
		y 31
	)
)
