;;; Sierra Script 1.0 - (do not remove this comment)
(script# 210)
(include sci.sh)
(include game.sh)
(include 210.shm)
(include 210.shp)
(use Cycle)
(use Door)
(use Feature)
(use Game)
(use Main)
(use System)
(use Polygon)
(use View)
(use Talker)
(use SpeakWindow)
(use Reverse)
(use Oscillate)
(use Sound)
(use Scaler)
(public
	rm210 0
	flowerTalker 1
	Speak-O-Tron 2
	wormTalker 3
)
(define NEAR_SCALE_PERCENT 100)       ; 100 is full size for the Scaler class.
(define FAR_SCALE_PERCENT 50)
(define NEAR_Y 121)
(define FAR_Y 91)
(define NORTH 209)
(define WEST 500)
(define EAST 600)
(local
	FourthTalk =  FALSE
	WormUp =  FALSE
	RoomExit =  0
)
; This rooms shows how to use Talkers and Narrators.

; A Narrator displays a series of messages in a row for a certain talker number.
; In the main.sc script, there is a switch statement that maps a talker number to a particular
; class in a particular script. The Narrator thus must be a public export.
; Narrators are often used in conjunction with the global SpeakWindow, which draws a speech bubble
; around the text.

; A Talker extends a narrator by providing a close up view of the talker's bust/face/torso, along with
; moving mouth and eyes.
(instance rm210 of Room
	(properties
		picture scriptNumber
		noun N_ROOM
	)
	
	(method (init)
		(AddPolygonsToRoom @P_Default210)
		; The flower hit target is also a barred area:
		(gRoom addObstacle: (CreateNewPolygon @P_Flower))
		(super init:)
		(self setScript: RoomScript)
		(switch gPreviousRoomNumber
			(else  (SetUpEgo -1 4) (gEgo posn: 231 97))
			; Set up ego view and loop (direction)
			(WEST (SetUpEgo -1 0) (gEgo posn: 27 138))
			(EAST (SetUpEgo -1 1) (gEgo posn: 280 138))
		)
		(gEgo
			init:
			; Set up the ego's scaling:
			setScale:
				Scaler
				NEAR_SCALE_PERCENT
				FAR_SCALE_PERCENT
				NEAR_Y
				FAR_Y
		)
		(theComputer
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Computer)
			approachVerbs: V_DO V_TALK
			init:
		)
		(theFlower
			setOnMeCheck: omcPOLYGON (CreateNewPolygon @P_Flower)
			approachVerbs: V_DO
			init:
		)
		(worm approachVerbs: V_DO V_TALK init: hide:)
	)
)

(instance RoomScript of Script
	(properties)
	
	(method (doit)
		(super doit:)
		(if FourthTalk
			(cond 
				((& (gEgo onControl:) ctlNAVY)
					(if (not WormUp)
						(= WormUp TRUE)
						(worm setMotion: MoveTo 262 105)
					)
				)
				(WormUp (= WormUp FALSE) (worm setMotion: MoveTo 262 119))
			)
		)
		(if (& (gEgo onControl:) ctlTEAL)
			(rm210 setScript: ExitWestScript)
		)
		(if (& (gEgo onControl:) ctlGREEN)
			(rm210 setScript: ExitNorthScript)
		)
		(if (& (gEgo onControl:) ctlMAROON)
			(rm210 setScript: ExitEastScript)
		)
	)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0)
			; Handle state changes
			(1
				(gMessager say: N_MACHINE V_TALK C_FIRSTTALK 0 self)
			)
			(2)
			; And wait for cue...
			; second batch
			(3
				(gGame handsOff:)
				(gMessager say: N_MACHINE V_TALK C_SECONDTALK 0 self)
			)
			(4 (worm setScript: wormDropScript self))
			(5 (gMessager say: N_WORM 0 C_SECONDTALK 0 self))
			; Worm can talk now.
			(6 (worm setScript: wormRiseScript self))
			(7 (gGame handsOn:))
			; and wait for the next cue
			; third batch
			(8
				(gGame handsOff:)
				(gMessager say: N_MACHINE V_TALK C_THIRDTALK 0 self)
			)
			(9 (worm setScript: wormDropScript self))
			(10 (gMessager say: N_WORM 0 C_THIRDTALK 1 self))
			; Worm can talk now.
			(11 (worm setCycle: CycleTo 3 -1 self))
			; And it stays out...
			(12
				(gGame handsOn:)
				(self cue:)
				(= FourthTalk TRUE)
				(= WormUp FALSE)
			)
			; Final batches...
			(13)
			; And wait for cue...
			(14
				(gMessager say: N_MACHINE V_TALK C_FOURTHTALK 0 self)
			)
			(15 (self changeState: (- state 2)))
		)
	)
)
                         ; We repeat now...
(instance ExitNorthScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(= RoomExit NORTH)
				(gGame handsOff:)
				(gEgo setMotion: MoveTo 179 86 self)
			)
			(1 (client setScript: ExitScript))
		)
	)
)

(instance ExitEastScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(= RoomExit EAST)
				(gGame handsOff:)
				(gEgo setMotion: MoveTo 340 (gEgo y?) self)
			)
			(1 (client setScript: ExitScript))
		)
	)
)

(instance ExitWestScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(= RoomExit WEST)
				(gGame handsOff:)
				(gEgo setMotion: MoveTo -20 (gEgo y?) self)
			)
			(1 (client setScript: ExitScript))
		)
	)
)

(instance ExitScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0
				(if FourthTalk
					(gGame handsOff:)
					(gMessager say: N_AFTERWARDS 0 0 0 self)
				else
					(gEgo setScale: 0)
					(gGame handsOn:)
					(gRoom newRoom: RoomExit)
				)
			)
			(1 (worm setScript: wormRiseScript self))
			(2
				(gEgo setScale: 0)
				(gGame handsOn:)
				(gRoom newRoom: RoomExit)
			)
		)
	)
)

(instance theComputer of Feature
	(properties
		noun N_MACHINE
		x 131        ; This is where the ego looks at
		y 72
		approachY 113      ; This is where he walks to
		approachX 131
		approachDist 10
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_TALK (RoomScript cue:))
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance wormDropScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (worm show:) (worm setMotion: MoveTo 262 119 self))
			(1 (worm setCycle: EndLoop self))
			(2 (self dispose:))
		)
	)
)

(instance wormRiseScript of Script
	(properties)
	
	(method (changeState newState)
		(= state newState)
		(switch state
			(0 (worm setCycle: BegLoop self))
			(1 (worm setMotion: MoveTo 262 91 self))
			(2 (worm hide:) (self dispose:))
		)
	)
)

(instance theFlower of Prop
	(properties
		view 210
		x 62
		y 172
		noun N_FLOWER
		signal $5010
	)
	
	(method (doVerb theVerb)
		(switch theVerb
			(V_DO (self setCycle: Oscillate 5))
			(else  (super doVerb: theVerb &rest))
		)
	)
)

(instance flowerTalker of Narrator
	(properties
		talkWidth 120
	)
	
	(method (init)
		(= font gFont)
		(= gWindow SpeakWindow)
		(gWindow tailX: 85 tailY: 135 xOffset: 15 thought: FALSE)          ; Offset from tail.
		(super init: &rest)
	)
	
	(method (dispose)
		(= gWindow gSq5Win_2)
		(super dispose: &rest)
	)
)

(instance Speak-O-Tron of Talker
	(properties
		x 10
		y 25
		view 1004
		loop 0
		talkWidth 150
		back 5
		textX 120
		textY 10
	)
	
	(method (init)
		(= font gFont)
		(computerVoice loop: -1 play:)
		; For talkers, we can optionally set a cycler on the eyes.
		; If there is no cycler, the Blink cycler will be used.
		(computerEyes setCycle: Reverse)
		(super
			init: computerBust computerEyes computerMouth &rest
		)
	)
	
	(method (dispose)
		; Note: in order for stop to actually stop notes (instead of just letting them play), a sound requires
		; channel 15 with the proper pre-amble.
		(computerVoice stop:)
		(super dispose: &rest)
	)
)

(instance computerBust of Prop
	(properties
		view 1004
	)
)

(instance computerEyes of Prop
	(properties
		cycleSpeed 10
		nsTop 0
		nsLeft 0
		view 1004
		loop 1
	)
)

(instance computerMouth of Prop
	(properties
		nsTop 59
		nsLeft 18
		view 1004
		loop 2
	)
)

(instance worm of Actor
	(properties
		view 211
		x 262
		y 91
		z 50
		approachX 262
		approachY 120
		noun N_WORM
		moveSpeed 3
		signal ignAct
	)
)

(instance wormTalker of Narrator
	(properties
		talkWidth 120
	)
	
	(method (init)
		(= font gFont)
		(= gWindow SpeakWindow)
		(gWindow tailX: 220 tailY: 41 xOffset: -50 thought: FALSE)           ; Offset from tail.
		(super init: &rest)
	)
	
	(method (dispose)
		(= gWindow gSq5Win_2)
		(super dispose: &rest)
	)
)

(instance computerVoice of Sound
	(properties
		number 210
		; number 101
		priority 15
		flags 1
	)
)
