;;; Sierra Script 1.0 - (do not remove this comment)
(script# 500)
(include sci.sh)
(include game.sh)
(include 500.shm)
(include 500.shp)
(use Controls)
(use Cycle)
(use Door)
(use Feature)
(use Game)
(use Main)
(use System)
(use Polygon)
(use Sound)
(use View)
(use ScaleTo)
(use Oscillate)
(public
	rm500 0
)
(define DECAY 1)
(define MIN_AMPLITUDE 1)
(define VOL_0_X 290)
(define VOL_100_X 30)
(local
	[AmplitudeToCycleSpeed 10] = [3 2 2 2 1 1 1 0 0 0]
	[Screens 3]
	NextScreen =  0
	PrevSignal =  1
)
; Controls the sine wave on the screen
; Amplitude is equal to number of cels at the start.
; Specified
; Full on
; Flip it
; Move it back if necessary
(procedure (GetSecondaryVol &tmp secondaryVol)
	(= secondaryVol (- (gEgo x?) VOL_100_X))
	(= secondaryVol
		(/ (* secondaryVol 100) (- VOL_0_X VOL_100_X))
	)
	(= secondaryVol (- 100 secondaryVol))
	(= secondaryVol (Max 0 secondaryVol))
	(= secondaryVol (Min 100 secondaryVol))
	(return secondaryVol)
)

(instance rm500 of Room
	(properties
		picture scriptNumber
		north 0
		east 210
		south 0
		west 0
		noun N_ROOM
	)
	
	(method (init)
		(AddPolygonsToRoom @P_Default500)
		(self setRegions: FLOPPYDISK_REGION)
		(super init:)
		(SetUpEgo)
		(switch gPreviousRoomNumber
			(else  (gEgo posn: 290 175 loop: 1))
		)
		(gEgo init:)
		(self setScript: RoomScript)
		(screen1
			init:
			setCycle: DecayOsc
			setPri: 0
			setMotion: MoveTo 320 (screen1 y?)
		)
		(screen2
			init:
			setCycle: DecayOsc
			setPri: 0
			setMotion: MoveTo 320 (screen2 y?)
		)
		(screen3
			init:
			setCycle: DecayOsc
			setPri: 0
			setMotion: MoveTo 320 (screen3 y?)
		)
		(= [Screens  0] screen1)
		(= [Screens  1] screen2)
		(= [Screens  2] screen3)
	)
)

(class DecayOsc of Cycle
	(properties
		client 0
		caller 0
		cycleDir 1
		cycleCnt 0
		completed 0
		amplitude 0
		xOrig 0
		quarterWidth 0
	)
	
	(method (init theClient theCaller)
		(= xOrig (theClient x?))
		(= quarterWidth
			(/
				(CelWide
					(theClient view?)
					(theClient loop?)
					(theClient cel?)
				)
				4
			)
		)
		(if (>= argc 2) (= caller theCaller))
		(super init: theClient)
		(self reset: MIN_AMPLITUDE)
	)
	
	(method (reset theAmplitude)
		(if (> argc 0)
			(= amplitude theAmplitude)
		else
			(= amplitude (client lastCel:))
		)
		(client cycleSpeed: [AmplitudeToCycleSpeed amplitude])
	)
	
	(method (doit &tmp oscNextCel repeatWidth)
		(= oscNextCel (self nextCel:))
		(if (or (> oscNextCel amplitude) (< oscNextCel 0))
			(= cycleDir (- cycleDir))
			(self cycleDone:)
			(if (< oscNextCel 0)
				(client posn: (+ (client x?) quarterWidth) (client y?))
			)
		else
			(client cel: oscNextCel)
		)
		(= repeatWidth (* 2 quarterWidth))
		(if (> (client x?) (+ xOrig repeatWidth))
			(client posn: (- (client x?) repeatWidth) (client y?))
		)
	)
	
	(method (cycleDone)
		(= amplitude (- amplitude DECAY))
		(= amplitude (Max MIN_AMPLITUDE amplitude))
		(client cycleSpeed: [AmplitudeToCycleSpeed amplitude])
		(client cel: (Min amplitude (self nextCel:)))
	)
)

(instance RoomScript of Script
	(properties)
	
	(method (doit &tmp newSignal)
		(= newSignal (gMusic1 prevSignal?))
		(super doit:)
		(if (and (== -1 newSignal) (!= newSignal PrevSignal))
			; Time to restart. This is our "manual looping", based on prevSignal changing and being -1
			(gMusic1 stop:)
			(self changeState: 0)
		)
		(= PrevSignal newSignal)
		(gMusic2 setVol: (GetSecondaryVol))
	)
	
	(method (changeState newState &tmp theScreen)
		(= state newState)
		(switch state
			(0
				; We are looping sounds, but we don't use the typical looping support (loop(-1)), since
				; we need to keep two separate sounds in sync (and they tend to drift if they're set to auto-loop)).
				(gMusic1 loop: 1 number: 500 play: self)
				(gMusic2
					loop: 1
					number: 501
					play: self
					setVol: (GetSecondaryVol)
				)
				(self changeState: 1)
			)
			(1)
			(2
				; Note: Regular cue points don't trigger if the sound has a loop point specified.
				; Also, -1 indicates the cue that the sound has ended. This only happens for non-looped sounds.
				(if (!= -1 (gMusic1 prevSignal?))
					(= theScreen [Screens NextScreen])
					((theScreen cycler?) reset:)
					(self changeState: 1)
					(++ NextScreen)
					(mod= NextScreen 3)
				)
			)
		)
	)
)

(instance screen1 of Actor
	(properties
		x 213
		y 64
		view 500
		signal ignAct
		noun N_WAVE
	)
)

(instance screen2 of Actor
	(properties
		x 12
		y 75
		view 500
		signal ignAct
		noun N_WAVE
	)
)

(instance screen3 of Actor
	(properties
		x 102
		y 88
		view 500
		signal ignAct
		noun N_WAVE
	)
)
